﻿using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WPFFlexGridSparklines.Model
{
    public class StockPriceData : INotifyPropertyChanged
    {
        private ObservableCollection<double> _priceData;
        private double _currentAmount;
        private double _currentPrice;
        private double _returns;

        public event PropertyChangedEventHandler? PropertyChanged;

        public string Name { get; private set; }
        public double InvestedAmount { get; private set; }
        public int Quantity { get; private set; }

        public double AverageBuyPrice
        {
            get
            {
                if (Quantity == 0 || InvestedAmount == 0)
                    return 0;

                return InvestedAmount / Quantity;
            }
        }

        public IEnumerable<double> PriceData
        {
            get 
            { 
                return _priceData; 
            }
        }

        public double CurrentAmount
        {
            get
            {
                return _currentAmount;
            }
            private set
            {
                _currentAmount = value;
                RaisePropertyChanged(nameof(CurrentAmount));
            }
        }

        public double CurrentPrice
        {
            get
            {
                return _currentPrice;
            }
            set
            {
                _currentPrice = value;
                if (_priceData.Count >= 10)
                    _priceData.RemoveAt(0);
                _priceData.Add(value);
                CurrentAmount = Quantity * value;
                Returns = CurrentAmount - InvestedAmount;
                RaisePropertyChanged(nameof(CurrentPrice));
            }
        }

        public double Returns
        {
            get
            {
                return _returns;
            }
            set
            {
                _returns = value;
                RaisePropertyChanged(nameof(Returns));
            }
        }

        public StockPriceData(string name, double currentPrice, int quantity)
        {
            _priceData = new ObservableCollection<double>();
            Name = name;
            Quantity = quantity;
            InvestedAmount = currentPrice * quantity;
            CurrentPrice = currentPrice;
        }

        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
